""" This module contains low level device drivers for the MPI probe station.

    These classes provide an interface to the native GPIB drivers from ADLINK and NI. There is no documentation
    available since this in an internal implementation detail.

    @private
"""
